/* -*- C++ -*-
 *
 * RemoteLocalReplicaCatalog.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 * $Id: RemoteLocalReplicaCatalog.h,v 1.50 2004/05/07 14:46:16 jamesc Exp $
 *
 */


#ifndef RemoteLocalReplicaCatalog_H
#define RemoteLocalReplicaCatalog_H

#include <string>
#include <vector>
#include <memory>
#include <iostream>
 
#include "EdgReplicaLocationService/LocalReplicaCatalog.h"
#include "EdgReplicaLocationService/ReplicationExceptions.h"

struct soap;
struct SOAP_ENV_Fault;

namespace EdgReplicaLocationService
{
  /** 
   *
   * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
   * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
   * @version $Id: RemoteLocalReplicaCatalog.h,v 1.50 2004/05/07 14:46:16 jamesc Exp $
   */

  class RemoteLocalReplicaCatalog : public LocalReplicaCatalog
  {
  
  public:

    /* create a new RemoteLocalReplicaCatalog
     * @param url     The URL of the LocalReplicaCatalog to connect to.  This
     *                will normally be a normal http/s URI which points 
     *                to a SOAP endpoint.

     * @param timeOut the timeout value, positive means seconds
                      negative microseconds,
     *                default is 30 seconds
     */
    RemoteLocalReplicaCatalog(const std::string& url,
                              int timeOut=30);

    /** standard virtual destructor
     */
    virtual ~RemoteLocalReplicaCatalog();

    /** get the URL this LocalReplicaCatalog resides at
     *
     * @return the URL
     */
    const std::string& getUrl() const;

    /// Impl of LocalReplicaCatalog

    /** add a new mapping to the Catalog.  
     * If there is already a mapping, 
     * we add this mapping to the set of mappings.  If there are no mappings 
     * already, we create a new mapping.
     *   
     * @param guid The guid
     * @param pfn The physical filename
     * @throws ValueTooLongException if the guid & pfn are longer then
     *         column size in the DB.
     *         @see ColumnSizes and
     *         @see getColumnSize( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void addMapping(const std::string& guid,
                            const std::string& pfn)
      throw(ValueTooLongException,
            PfnExistsException,
            CommunicationException);
                        
    /** delete all parts of an guid to pfn mapping including attached
     * attributes.
     *
     * @param guid The guid for which to delete the mapping
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void deleteMapping(const std::string& guid)
      throw(CommunicationException);

    /** remove from the catalog a single guid to pfn mapping.
     *
     * @param guid the guid whose mapping we look at
     * @param pfn the physical filename to remove
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void removeMapping(const std::string& guid,
                               const std::string& pfn)
      throw(CommunicationException);

    /** Do we have a given guid in the catalog?
     *
     * @param guid The guid we are checking for
     * @return <B>true</B> if we have a mapping for the guid, 
     * <B>false</B> otherwise
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool guidExists(const std::string& guid) const
      throw(CommunicationException);
                        
    /** Do we have a given physical filename in the catalog?
     *
     * @param pfn The physical filename we are checking for
     * @return <B>true</B> if we have a mapping for the physical filename, 
     * <B>false</B> otherwise
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool pfnExists(const std::string& pfn) const
      throw(CommunicationException);
    
    /** For a given guid, return all the physical filenames we map 
     * to in this Catalog.
     * This will always have at least one entry, and never be empty.
     * @param guid The GUID
     * @param output the vector in which to return the set of Physical FileNames
     * @throws NoSuchGuidException If the guid does not exist in the catalog.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void getPfns(const std::string&        guid,
                         std::vector<std::string>& output) const
      throw(NoSuchGuidException,
            CommunicationException);
  
    /** For a given pfn, return the guid that points to it.
     * The return value will never be <B>null</B>.
     * @param pfn The physical filename.
     * @return the guid that maps to it
     * @throws NoSuchPfnException If the pfn does not exist in the
     *         catalog.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string guidForPfn(const std::string& pfn) const
      throw(NoSuchPfnException,
            CommunicationException);

    /** Create a new Attribute Definition in the Catalog.
     *
     * @param  attrDef The new atttribute definition object
     * @see    AttributeDefinition
     * @throws AttributeAlreadyExistsException if an attribute with
     *         the given name already exists
     * @throws ValueTooLongException if the attrName is longer then
     *         column size in the DB.
     * @see    ColumnSizes
     * @see    getColumnSize( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    createAttributeDefinition(const AttributeDefinition& attrDef)
      throw(AttributeDefinitionExistsException,
            EmptyAttributeNameException,
            ValueTooLongException,
            CommunicationException);

    /** Get the AttributeDefinition corresponding to the attribute name.
     *
     * @param attrName the attribute name
     * @throws NoSuchAttributeDefinitionException if an attribute with
     *         the given name doesn't exist
     */
    virtual AttributeDefinition
    getAttributeDefinition(const std::string& attrName) const
      throw(NoSuchAttributeDefinitionException,
            CommunicationException);

    /** Retrieve the list of all availabel attribute definitions
     *
     * @param attdefs the reference to a vector to be filled by the
     *                attribute definition names
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getAttributeDefinitions(AttrDefs_t& attdefs) const
      throw(CommunicationException);

    /** Delete an Attribute Definition from the Catalog.
     * This will also remove any attribute value with this definition from
     * GUID and Physical FileName entries in the Catalog.
     *
     * @param attrName the name of the attribute definition to remove
     * @throws NoSuchAttributeDefinitionException if an attribute with
     *         the given name does not exist
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    removeAttributeDefinition(const std::string& attrName)
      throw(NoSuchAttributeDefinitionException,
            EmptyAttributeNameException,
            CommunicationException);

    /** do we have an attribute definition for a given attribute name?
     *
     * @param attrName the name of attribute to check for
     * @return <b>true</b> if the attribute name is there.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool attributeDefinitionExists(const std::string& attrName) const
      throw(CommunicationException);
                
    /** Add an attribute value to a PFN.
     *
     * @param pfn The PFN
     * @param attrName the name of attribute
     * @param value The attribute value we'll add. 
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the definition of
     *         the attribute does not exist in the Catalog.
     * @throws ValueTooLongException if the attrName is longer then
     *         column size in the DB.
     *         @see ColumnSizes and
     *         @see getColumnSize( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     DEPRECATED
    */
    virtual void setPfnAttribute(const std::string& pfn, 
                                 const std::string& attrName,
                                 const std::string& value)
      throw(NoSuchPfnException,
            NoSuchAttributeDefinitionException,
            ValueTooLongException,
            CommunicationException);
      
    /** Add an attribute value to a PFN attribute of type string.
     *
     * @param pfnName The PFN
     * @param attrDefnName the name of attribute
     * @param attrValue The attribute value we'll add. 
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the definition of
     *         the attribute does not exist in the Catalog.
     * @throws UnappropriateAttributeTypeException
     *         if the attribute is not of type string
     */
    virtual void
    setStringPfnAttribute( const std::string& pfnName,
                           const std::string& attrDefnName,
                           const std::string& attrValue)
      throw(NoSuchAttributeDefinitionException,
            NoSuchPfnException,
            UnappropriateAttributeTypeException,
            CommunicationException);
      
    /** Add an attribute value to a PFN attribute of type int.
     *
     * @param pfnName The PFN
     * @param attrDefnName the name of attribute
     * @param attrValue The attribute value we'll add. 
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the definition of
     *         the attribute does not exist in the Catalog.
     * @throws UnappropriateAttributeTypeException
     *         if the attribute is not of type int
     */
    virtual void
    setIntPfnAttribute( const std::string& pfnName,
                        const std::string& attrDefnName,
                        int attrValue)
      throw(NoSuchAttributeDefinitionException,
            NoSuchPfnException,
            UnappropriateAttributeTypeException,
            CommunicationException);
      
    /** Add an attribute value to a PFN attribute of type float.
     *
     * @param pfnName The PFN
     * @param attrDefnName the name of attribute
     * @param attrValue The attribute value we'll add. 
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the definition of
     *         the attribute does not exist in the Catalog.
     * @throws UnappropriateAttributeTypeException
     *         if the attribute is not of type float
     */
    virtual void
    setFloatPfnAttribute(const std::string& pfnName,
                         const std::string attrDefnName,
                         float attrValue)
      throw(NoSuchAttributeDefinitionException,
            NoSuchPfnException,
            UnappropriateAttributeTypeException,
            CommunicationException);

    //       /** Add an attribute value to a PFN attribute of type date.
    //        *
    //        * @param pfnName The PFN
    //        * @param attrDefnName the name of attribute
    //        * @param attrValue The attribute value we'll add. 
    //        * @throws NoSuchPfnException if the PFN does not exist in the Catalog
    //        * @throws NoSuchAttributeDefinitionException if the definition of
    //        *         the attribute does not exist in the Catalog.
    //        * @throws UnappropriateAttributeTypeException
    //        *         if the attribute is not of type date
    //        */
    // TO BE REVISITED
    //       virtual void setDatePfnAttribute( const std::string& pfnName,
    //                                         const std::string& attrDefnName,
    //                                         Date attrValue)
    //         throw(NoSuchAttributeDefinitionException,
    //               NoSuchPfnException,
    //               UnappropriateAttributeTypeException,
    //               CommunicationException);
      
    /** return the value of a given attribute for an PFN.
     *
     * @param pfn the PFN
     * @param attrName the AttributeDefinition name to check for
     * @return the attribute value
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the AttributeDefintion
     *         does not exist in the catalog.
     * @throws CommunicationException in case of an unknown server error
     DEPRECATED
    */

    virtual const std::string getPfnAttribute(const std::string& pfn, 
                                              const std::string& attrName) const
      throw(NoSuchPfnException,
            NoSuchAttributeDefinitionException,
            CommunicationException);
      
    /** return the value of a given string attribute for a PFN.
     *
     * @param pfnName the PFN
     * @param attrDefnName the AttributeDefinition name to check for
     * @return the attribute value
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the AttributeDefintion
     *         does not exist in the catalog.
     */
    virtual std::string
    getStringPfnAttribute( const std::string& pfnName,
                           const std::string& attrDefnName)
      throw(NoSuchPfnException,
            NoSuchAttributeDefinitionException,
            UnappropriateAttributeTypeException,
            CommunicationException);
      
    /** return the value of a given int attribute for an PFN.
     *
     * @param pfnName the PFN
     * @param attrDefnName the AttributeDefinition name to check for
     * @return the attribute value
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the AttributeDefintion
     *         does not exist in the catalog.
     */
    virtual int
    getIntPfnAttribute( const std::string& pfnName,
                        const std::string& attrDefnName)
      throw(NoSuchPfnException,
            NoSuchAttributeDefinitionException,
            UnappropriateAttributeTypeException,
            CommunicationException);
      
    /** return the value of a given float attribute for an PFN.
     *
     * @param pfnName the PFN
     * @param attrDefnName the AttributeDefinition name to check for
     * @return the attribute value
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the AttributeDefintion
     *         does not exist in the catalog.
     */
    virtual float
    getFloatPfnAttribute( const std::string& pfnName,
                          const std::string& attrDefnName)
      throw(NoSuchPfnException,
            NoSuchAttributeDefinitionException,
            UnappropriateAttributeTypeException,
            CommunicationException);

    //       /** return the value of a given date attribute for an PFN.
    //        *
    //        * @param pfnName the PFN
    //        * @param attrDefnName the AttributeDefinition name to check for
    //        * @return the attribute value
    //        * @throws NoSuchPfnException if the PFN does not exist in the Catalog
    //        * @throws NoSuchAttributeDefinitionException if the AttributeDefintion
    //        *         does not exist in the catalog.
    //        */
    // TO BE REVISITED
    //       virtual Date getDatePfnAttribute( const std::string& pfnName,
    //                                         const std::string& attrDefnName)
    //         throw(NoSuchPfnException,
    //               NoSuchAttributeDefinitionException,
    //               UnappropriateAttributeTypeException
    //               CommunicationException);

    /** remove the value of a given attribute for an PFN.
     *
     * @param  pfn the PFN
     * @param  attrName the AttributeDefinition name to check for
     * @throws NoSuchPfnException if the PFN does not exist in the Catalog
     * @throws NoSuchAttributeDefinitionException if the AttributeDefintion
     *         does not exist in the catalog.
     * @throws NoSuchAttributeException if there is no such attribute for the
     *         given PFN
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void removePfnAttribute(const std::string& pfn, 
                                    const std::string& attrName)
      throw(NoSuchPfnException,
            NoSuchAttributeDefinitionException, NoSuchAttributeException,
            CommunicationException);
   
    /** Set the number of mappings that will be displayed by default.
     * (for the getMappingsByGuid and getMappingsByPfn methods)..
     *
     * @param  resultLength the number of mappings that will be displayed
     *         by default
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void setDefaultResultLength(int resultLength)
      throw(CommunicationException);

    /** Return the number of mappings that will be displayed by default.
     * (for the getMappingsByGuid and getMappingsByPfn methods).
     *
     * @return the number of mappings displayed by default.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual int getDefaultResultLength() const
      throw(CommunicationException);

    /** Return the maximum number of characters for a GUID, a PFN and an attribute.
     *  (according to the schema : if GUID has been defined as
     *  VARCHAR(250), then the GUID cannot have more than 250 characters. 
     *  So 250 will be returned.)
     *
     * @return the object containing the Column sizes for Attr. def., GUID, PFN
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void getColumnSizes( ColumnSizes& cs ) const
      throw(CommunicationException);

    /** Return the mappings for a list of guids.  If some of the guids do not
     * exist, this is not a problem.  If there are no results, an empty array
     * is returned
     * @param guids the list of guids, which may be empty, but may not be
     * <code>null</code>
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     */
    virtual void
    getMappingsWithAttributes(const std::vector<std::string>& guids,
                              std::vector<Mapping>& mappings) const
      throw(CommunicationException);

    /** Add a set of guid to pfn mappings, with optional pfn attributes to
     * the database.  If any problem occurs when inserting entries or
     * attributes (pfn already exists, inserting an attribute value for an
     * attribute which doesn't exist, inserting the wrong type of attribute
     * value) the entire transaction is rolled back, and a fault thrown
     *
     * @param mappings a vector of GuidPfnPair_t
     */
    virtual void 
    setMappingsWithAttributes(const std::vector<Mapping>& mappings) const
      throw(CommunicationException, NoSuchAttributeDefinitionException,
             UnappropriateAttributeTypeException, PfnExistsException);

    /** Return the guid to physical filename mappings which contain a
     * guid that matches the given guidPattern.
     * We assume unix regexp semantics.  The result will be an empty set
     * if no mapppings are found.
     *
     * @param guidPattern The GUID pattern we are searching for
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuid(const std::string& guidPattern,
                      std::vector<GuidPfnPair_t>& mappings) const
      throw(CommunicationException);

    /** Return the guid to physical filename mappings which contain a guid
     * that matches the given guidPattern. Return only the first mappings 
     * (no more than the resultLength parameter).
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param guidPattern The GUID pattern we are searching for
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuid(const std::string&          guidPattern,
                      int                         resultLength,
                      std::vector<GuidPfnPair_t>& mappings    ) const
      throw(CommunicationException);
    
    /** Return the guid to physical filename mappings which contain a guid
     * that matches the given guidPattern. Return only a certain number of mappings.
     * Return We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param guidPattern The GUID pattern we are searching for
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuid(const std::string&          guidPattern,
                      int                         resultLength,
                      int                         resultStart,
                      std::vector<GuidPfnPair_t>& mappings    ) const
      throw(CommunicationException);


    /** Return the guid to physical filename mappings which contain a
     * physical filename that matches the given pfnPattern.
     * We assume unix regexp semantics.  The result will be an empty set
     * if no mapppings are found.
     *
     * @param pfnPattern The Physical FileName pattern we are searching for
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByPfn(const std::string&          pfnPattern,
                     std::vector<GuidPfnPair_t>& mappings) const
      throw(CommunicationException); 

    /** Return the guid to physical filename mappings which contain a pfn
     * that matches the given pfnPattern. Return only the first mappings 
     * (no more than the resultLength parameter).
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param pfnPattern The PFN pattern we are searching for
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByPfn(const std::string&          pfnPattern,
                     int                         resultLength,
                     std::vector<GuidPfnPair_t>& mappings) const
      throw(CommunicationException);
    
    /** Return the guid to physical filename mappings which contain a pfn
     * that matches the given pfnPattern. Return only a certain number of mappings.
     * Return We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param pfnPattern The PFN pattern we are searching for
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByPfn(const std::string&          pfnPattern,
                     int                         resultLength,
                     int                         resultStart,
                     std::vector<GuidPfnPair_t>& mappings) const
      throw(CommunicationException);
    
    /** Return the guid to physical filename mappings which contain
     * the PFNs corresponding to the given attributes conditions.
     * Return only the first mappings (no more than the resultLength parameter).
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param SQLQuery The attributes conditions
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings The mappings to be filled into
     */
    virtual void
    getMappingsByAttribute( const std::string& SQLQuery, 
                            int resultLength,
                            std::vector<GuidPfnPair_t>& mappings) const
      throw(InvalidQueryException,
            CommunicationException);
    
    /** Return the guid to physical filename mappings which 
     * the PFNs corresponding to the given attributes conditions.
     * Return only a certain number of mappings.
     * Return We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param SQLQuery The attributes conditions
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings The mappings to be filled into
     */
    virtual void
    getMappingsByAttribute( const std::string& SQLQuery, 
                            int resultLength, 
                            int resultStart,
                            std::vector<GuidPfnPair_t>& mappings) const
      throw(InvalidQueryException,
            CommunicationException);

    /** Return the guid to physical filename mappings which the PFNs
     * corresponding to the given conditions, along with their attributes.

     * Return only a certain number of mappings.
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param SQLQuery The attributes conditions
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings The mappings to be filled into
     */
    virtual void
    getMappingsWithAttributes( const std::string& SQLQuery, 
                               int resultLength,
                               std::vector<Mapping>& mappings) const
      throw(InvalidQueryException, CommunicationException);

    /** Return the guid to physical filename mappings which the PFNs
     * corresponding to the given conditions, along with their attributes.

     * Return only a certain number of mappings.
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param SQLQuery The attributes conditions
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings The mappings to be filled into
     */
    virtual void
    getMappingsWithAttributes( const std::string& SQLQuery, 
                               int resultLength, 
                               int resultStart,
                               std::vector<Mapping>& mappings) const
      throw(InvalidQueryException,  CommunicationException);


    /** Return the guid to physical filename mappings which have a given 
     * attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     * We assume unix regexp semantics for valuePattern.
     *
     * @param attrName the name of the attribute
     * @param valuePattern the value of attribute to search for
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     DEPRECATED
    */

    virtual void
    getMappingsByPfnAttribute(const std::string& attrName,
                              const std::string& valuePattern,
                              std::vector<GuidPfnPair_t>& mappings) 
      throw(NoSuchAttributeDefinitionException,
            CommunicationException);

    /** Return the guid to physical filename mappings which have a given 
     * attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     * We assume unix regexp semantics for valuePattern.
     *
     * @param attrName the name of the attribute
     * @param valuePattern the value of attribute to search for
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     DEPRECATED
    */

    virtual void
    getMappingsByPfnAttribute(const std::string& attrName,
                              const std::string& valuePattern,
                              int                resultLength,
                              std::vector<GuidPfnPair_t>& mappings) 
      throw(NoSuchAttributeDefinitionException,
            CommunicationException);

    /** Return the guid to physical filename mappings which have a given 
     * attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     * We assume unix regexp semantics for valuePattern.
     *
     * @param attrName the name of the attribute
     * @param valuePattern the value of attribute to search for
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings pointer to a vector of GuidPfnPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     DEPRECATED
    */

    virtual void
    getMappingsByPfnAttribute(const std::string& attrName,
                              const std::string& valuePattern,
                              int                resultLength,
                              int                resultStart,
                              std::vector<GuidPfnPair_t>& mappings) 
      throw(NoSuchAttributeDefinitionException,
            CommunicationException);
                
    /** Add an <code>RLI URL</code> to the list of RLI subscribers. The lrc URL
     * is also passed so that the LRC servlet knows its own URL
     *
     * @param lrcURL The URL for the LRC servlet
     * @param rliURL The URL for the RLI subscriber
     * @throws CommunicationException in case of an unknown server error
     DEPRECATED
    */     

    virtual bool addRLISubscription(const std::string& lrcURL,
                                    const std::string& rliURL)
      throw(CommunicationException);

    /** Add an <code>RLI URL</code> to the list of RLI subscribers. 
     *
     * @param rliURL The URL for the RLI subscriber
     */     
    virtual bool addRLISubscription( const std::string& rliURL)
      throw( CommunicationException);

    /** Remove a <code>RLI URL</code> from the list of RLI subscribers.
     *
     * @param rliURL The URL for the RLI subscriber
     * @throws CommunicationException in case of an unknown server error
     */     
    virtual bool removeRLISubscription(const std::string& rliURL)
      throw(CommunicationException);

    /** Return the list of RLIs that currently subscribe to this LRC
     *
     * @param rlis variable where the current list of RLIs is returned
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void getRLISubscriptions(RliList_t& rlis) const
      throw(CommunicationException);
                 
    /** Force a bloom filter update to all the RLI subscribers
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void forceRLIUpdate() const
      throw(CommunicationException);
                        
    /** Check if a server is alive. It returns a stats string.
     * If everything is ok, this will start with <code>OK</code> else
     * it will start with <code>FAIL</code>.
     * @return the stats string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string ping() const
      throw(CommunicationException);

    /** Return RLS server version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getVersion() const
      throw(CommunicationException);
                
    /** Return RLS server interface version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getInterfaceVersion() const
      throw(CommunicationException);
                
    /** Return RLS server schema version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getSchemaVersion() const
      throw(CommunicationException);
                
  private:
      
    //
    // helper methods
    //

    void dump_soap_error(int ret, const struct soap* const_soap) const;

    void handleGenericFaults(int ret, const struct soap* soap) const;

    const char* endpoint() const;

    //
    // member variables
    //
    const std::string m_endpoint;
    struct soap *m_soap;

#ifdef WITH_WP2_SECURITY                        
    /* sorry, had to loosen the visibility of soap()
     * a bit. Could try protected and friend decls as well.
     */
  public:
#endif
    struct soap* soap() const;

#ifdef WITH_WP2_SECURITY
    /* For defining the desired role and policy
     * after the lrc object has laready been created
     */
    int addRoleAndPolicy(char *role,
                         char *policy,
                         char *defaultrole,
                         char *defaultpolicy);

    void clearRoleAndPolicy();

#endif
  };
}

#endif

// EOF RemoteLocalReplicaCatalog.h
