/* -*- C++ -*-
 *
 * ReplicationExceptions.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 * $Id: ReplicationExceptions.h,v 1.19 2003/09/22 06:16:04 rado Exp $
 *
 */

#include <exception>
#include <string>

#ifndef RLSReplicationExceptions_H
#define RLSReplicationExceptions_H 1

namespace EdgReplicaLocationService {
/** 
 *
 * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
 * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
 * @version $Id: ReplicationExceptions.h,v 1.19 2003/09/22 06:16:04 rado Exp $
 */

  class ReplicationException : public std::exception {
  /**
   * A simple top-level exception class for Replication Exceptions.
   *
   */

    public:

    public:
      ReplicationException() throw() 
       : m_faultCode("FAULTABLE") {
      }
      ReplicationException(const std::string& what) throw() 
       : m_what (what), m_faultCode("INTERNAL") {
      }
      ReplicationException(const std::string& what,
			   const std::string& faultCode) throw() 
       : m_what (what), m_faultCode(faultCode) {
      }
    const std::string& faultCode() const throw() {
      return m_faultCode;
    }
      virtual const char* what () const throw() {
        return m_what.c_str ();
      }
      virtual ~ReplicationException() throw(){
      }

    private:
      std::string m_what;
      std::string m_faultCode;

  };

  /**
   * An exception thrown when we don't implement a method yet
   */
  class NotImplementedException : public ReplicationException {
    public: 
      NotImplementedException(const std::string& method) 
      : ReplicationException("Not Implemented: " + method),
        m_method(method) {
      }
      virtual ~NotImplementedException() throw(){
      }
      const std::string& getMethod() const {
        return m_method;
      }
      
    private:
      const std::string m_method;
  };

  /**
   * A exception thrown when ... .
   */
  class EmptyAttributeNameException : public ReplicationException {
    public: 
    EmptyAttributeNameException(const std::string& message) 
      :  ReplicationException(message) {
    }
      virtual ~EmptyAttributeNameException() throw(){
      }
  };

  /**
   * A exception thrown when ... .
   */
  class NoSuchGuidException : public ReplicationException {
    public: 
      NoSuchGuidException(const std::string& guid) 
      : ReplicationException("No Such Guid: " + guid, "NOSUCHGUID"),
        m_guid(guid) {
      }
      virtual ~NoSuchGuidException() throw(){
      }
      const std::string& getGuid() const {
        return m_guid;
      }

    private:
      const std::string m_guid;
  };

  /**
   * A exception thrown when ... .
   */
  class NoSuchPfnException : public ReplicationException {
    public: 
      NoSuchPfnException(const std::string& pfn) 
      : ReplicationException("No Such Pfn: " + pfn, "NOSUCHPFN"),
        m_pfn(pfn) {
      }
      virtual ~NoSuchPfnException() throw(){
      }
      const std::string getPfn() const {
         return m_pfn;
      }

    private:
      const std::string m_pfn;
  };

  /**
   * A exception thrown when ... .
   */
  class PfnExistsException : public ReplicationException {
    public: 
      PfnExistsException(const std::string& pfn) 
      : ReplicationException("Pfn Exists: " + pfn, "PFNEXISTS"),
        m_pfn(pfn) {
      }
      virtual ~PfnExistsException() throw(){
      }
      const std::string getPfn() const {
        return m_pfn;
      }

    private:
      const std::string m_pfn;
  };

  /**
   * A exception thrown when ... .
   */
  class AttributeExistsException : public ReplicationException {
    public:  
      AttributeExistsException(const std::string& message) 
      : ReplicationException(message, "ATTRIBUTEEXISTS") {
      }
      virtual ~AttributeExistsException() throw() {
      }
  };

  /**
   * A exception thrown when ... .
   */
  class AttributeDefinitionExistsException : public ReplicationException {
    public: 
      AttributeDefinitionExistsException(const std::string& message) 
      : ReplicationException(message, "ATTRIBUTEDEFNEXISTS") {
      }
      virtual ~AttributeDefinitionExistsException() throw(){
      }
  };

  /**
   * A exception thrown when ... .
   */
  class NoSuchAttributeDefinitionException : public ReplicationException {
    public: 
      NoSuchAttributeDefinitionException(const std::string& message) 
      : ReplicationException(message, "NOSUCHATTRIBUTEDEFN") {
      }
      virtual ~NoSuchAttributeDefinitionException() throw(){
      }
  };

  /**
   * A exception thrown when ... .
   */
  class NoSuchAttributeException : public ReplicationException {
    public: 
      NoSuchAttributeException(const std::string& message) 
      : ReplicationException(message, "NOSUCHATTRIBUTE") {
      }
      virtual ~NoSuchAttributeException() throw(){
      }
  };

	/**
	 * An exception thrown when too long name is to be stored
	 */
	class ValueTooLongException : public ReplicationException {
		public:
			ValueTooLongException( const std::string& message )
	      : ReplicationException(message, "TOOLONGNAME") {
	    }
	    virtual ~ValueTooLongException() throw() {
	    }
	};

	/**
	 * An exception thrown when invalid attribute type is requested
	 */
	class UnappropriateAttributeTypeException : public ReplicationException {
		public:
			UnappropriateAttributeTypeException( const std::string& message )
	      : ReplicationException(message, "ATTRTYPENOTGOODFORATTRNAME") {
	    }
	    virtual ~UnappropriateAttributeTypeException() throw() {
	    }
	};
  
	/**
	 * An exception thrown when invalid query is applied
	 */
	class InvalidQueryException : public ReplicationException {
		public:
			InvalidQueryException( const std::string& message )
	      : ReplicationException(message, "INVALIDQUERY") {
	    }
	    virtual ~InvalidQueryException() throw() {
	    }
	};

  class CommunicationException : public ReplicationException {
    public: 
      CommunicationException() 
      : ReplicationException() {
      }
      CommunicationException(const std::string& message) 
      : ReplicationException(message, "CONNECTION") {
      }
      virtual ~CommunicationException() throw(){
      }
  };

  /** A dispatch method for the soap faults that will throw the
   * appropriate language exception
   * @param fault fault string, as defined by the LRC SOAP API
   * @param detail fault specific detail information
   */
  void
    throwReplicationException(const std::string& fault,
	                            const std::string& detail);
}


#endif // ReplicationExceptions_H

