/* -*- C++ -*- $Id: ReplicaMetadataCatalogImpl.h,v 1.25 2004/06/27 15:42:09 jamesc Exp $
 *
 * ReplicaMetadataCatalogImpl.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */


#ifndef ReplicaMetadataCatalogImpl_H
#define ReplicaMetadataCatalogImpl_H 1

#include "EdgReplicaMetadataCatalog/ReplicaMetadataCatalog.h"
#include "EdgReplicaMetadataCatalog/ReplicationExceptions.h"

#include <string>
#include <vector>
#include <memory>
#include <iostream>

struct soap;
struct SOAP_ENV_Fault;

namespace EdgReplicaMetadataCatalog {

  /** 
   *
   * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
   * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
   * @version $Id: ReplicaMetadataCatalogImpl.h,v 1.25 2004/06/27 15:42:09 jamesc Exp $
   */

  class ReplicaMetadataCatalogImpl : public ReplicaMetadataCatalog
  {
			
  public:
    /** create a new RemoteLocalReplicaCatalog
     * @param url  The URL of the LocalReplicaCatalog to connect to.
     * This will normally be a normal http/s URI which points
     * to a SOAP endpoint.
     * @param timeOut the time-out value, default is 30s, if positive it means seconds,
     *                if negative it means microseconds
     */
    ReplicaMetadataCatalogImpl(const std::string& url,
			       int timeOut);
#ifdef WITH_WP2_SECURITY

    ReplicaMetadataCatalogImpl(const std::string& url,
			       char *passphrase,
			       char *credentials,
			       char *cacertdir,
			       char *rseedfile,
			       int timeOut);
#endif	  

			
    virtual ~ReplicaMetadataCatalogImpl();
			
    const std::string& getUrl() const;
			
    //
    // Implementation of MetadataCatalog
    //
      
    /** Add an alias into catalog
     *
     * @param guid the guid name
     * @param alias the new alias name
     * @throws AliasExistsException if an alias already exists
     * @throws ValueTooLongException if the attrName is longer than
     *                               column size in DB schema
     * @see ColumnSizes
     * @see getColumnSizes( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    addAlias(const std::string& guid,
	     const std::string& alias)
      throw(AliasExistsException,
	    ValueTooLongException,
	    CommunicationException);

    /** Remove an alias  from the catalog
     *
     * @param guid the guid name
     * @param alias the alias name to be removed
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    removeAlias(const std::string& guid,
		const std::string& alias)
      throw(CommunicationException);

    /** Do we have a given alias in the catalog?
     *
     * @param alias The alias we are checking for
     * @return <B>true</B> if we have a mapping for the alias, 
     * <B>false</B> otherwise
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool
    aliasExists(const std::string& alias) const
      throw(CommunicationException);
			
    /** Do we have a given guid in the catalog?
     *
     * @param guid The guid we are checking for
     * @return <B>true</B> if we have a mapping for the guid, 
     * <B>false</B> otherwise
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool
    guidExists(const std::string& guid) const
      throw(CommunicationException);
			
    /** Find a corresponding guid for a given alias
     *
     * @param alias the alias name
     * @return the guid
     * @throws NoSuchAliasException
     *         if a guid with the given name does not exists
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string
    guidForAlias(const std::string&  alias) const
      throw(NoSuchAliasException,
	    CommunicationException);
			
    /** Retrieve the list of all aliases pointing to the same guid
     *
     * @param guid the guid
     * @param vec the vector of alias names gets filled if success,
     *            empty otherwise
     * @throws NoSuchGuidException
     *         if a guid with the given name does not exists
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getAliases(const std::string& guid,
	       Aliases_t& vec ) const
      throw(NoSuchGuidException,
	    CommunicationException);
			
    /** Create a new alias Attribute Definition in the Catalog.
     *
     * @param attrDef The name of the new alias atribute
     * @throws AttributeAlreadyExistsException if an attribute with
     *         the given name already exists
     * @throws EmptyAttributeNameException if an attrName is empty
     * @throws ValueTooLongException if the attrName is longer than
     *                               column size in DB schema
     * @see ColumnSizes
     * @see getColumnSizes( ColumnSizes & )
     * @see AttributeDefinition
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    createAliasAttributeDefinition(const AttributeDefinition& attrDef)
      throw(AttributeDefinitionExistsException,
	    UnsupportedAttributeTypeException,
	    EmptyAttributeNameException,
	    ValueTooLongException,
	    CommunicationException);
	  
	  
    virtual void
    createAliasAttributeDefinition(const std::string& attrName)
      throw(AttributeDefinitionExistsException,
	    UnsupportedAttributeTypeException,
	    EmptyAttributeNameException,
	    ValueTooLongException,
	    CommunicationException);
	  
    /** Retrieve the list of all available alias attribute definitions
     *
     * @param ads the reference to a vector to be filled by the alias
     *            attribute definition names
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getAliasAttributeDefinitions( AttrDefs_t& ads ) const
      throw(CommunicationException);
			
    /** Retrieve the alias attribute definition
     *
     * @param attrAliasName the attribute name
     * @throws CommunicationException in case of an unknown server error
     */
    virtual AttributeDefinition
    getAliasAttributeDefinition( const std::string& attrAliasName ) const
      throw(CommunicationException);
			
    /** do we have an alias attribute definition for a given attribute name?
     *
     * @param attrName the name of an alias attribute to check for
     * @return <b>true</b> if the attribute name is there.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool
    aliasAttributeDefinitionExists(const std::string& attrName) const
      throw(CommunicationException);
			
    /** Delete an alias Attribute Definition from the Catalog.
     * This will also remove any attribute value with
     * this definition from GUID and Physical FileName
     * entries in the Catalog.
     *
     * @param attrName the name of the alias attribute definition to remove
     * @throws NoSuchAttributeDefinitionException if an attribute definition
     *         with the given name does not exist
     * @throws EmptyAttributeNameException if an attrName is empty
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    removeAliasAttributeDefinition(const std::string& attrName)
      throw(NoSuchAttributeDefinitionException,
	    EmptyAttributeNameException,
	    CommunicationException);
			
    /** Create a new guid Attribute Definition in the Catalog.
     *
     * @param  attrDef The name of the new guid atribute
     * @throws AttributeAlreadyExistsException if an attribute with
     *         the given name already exists
     * @throws EmptyAttributeNameException if an attrName is empty
     * @throws ValueTooLongException if the attrName is longer than
     *                               column size in DB schema
     * @see ColumnSizes
     * @see getColumnSizes( ColumnSizes & )
     * @see AttributeDefinition
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    createGuidAttributeDefinition(const AttributeDefinition& attrDef)
      throw(AttributeDefinitionExistsException,
	    UnsupportedAttributeTypeException,
	    EmptyAttributeNameException,
	    ValueTooLongException,
	    CommunicationException);
			
    virtual void
    createGuidAttributeDefinition(const std::string& attrName)
      throw(AttributeDefinitionExistsException,
	    UnsupportedAttributeTypeException,
	    EmptyAttributeNameException,
	    ValueTooLongException,
	    CommunicationException);
	  
    /** Retrieve the list of all available guid attribute definitions
     *
     * @param ads the reference to a vector to be filled by the
     *            attribute definition names
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getGuidAttributeDefinitions( AttrDefs_t& ads ) const
      throw(CommunicationException);
			
    /** Retrieve the guid attribute definition
     *
     * @param attrGuidName the guid attribute name
     * @throws CommunicationException in case of an unknown server error
     */
    virtual AttributeDefinition
    getGuidAttributeDefinition( const std::string& attrGuidName ) const
      throw(CommunicationException);
			
    /** do we have an guid attribute definition for a given attribute name?
     *
     * @param attrName the name of attribute to check for
     * @return <b>true</b> if the attribute name is there.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual bool
    guidAttributeDefinitionExists(const std::string& attrName) const
      throw(CommunicationException);
			
    /** Delete a guid Attribute Definition from the Catalog.
     * This will also remove any attribute value with
     * this definition from GUID and Physical FileName
     * entries in the Catalog.
     *
     * @param attrName the name of the attribute definition to remove
     * @throws NoSuchAttributeDefinitionException if an attribute definition
     *         with the given name does not exist
     * @throws EmptyAttributeNameException if an attrName is empty
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    removeGuidAttributeDefinition(const std::string& attrName)
      throw(NoSuchAttributeDefinitionException,
	    EmptyAttributeNameException,
	    CommunicationException);
			
    /** Returns the old guid attribute, or <code>empty</code> string
     * if no old attribute exists
     *
     * @param guidName the guid name
     * @param attrDefnName the attribute definition name
     * @param attrValue the new attribute value
     * @return and old attribute value if exists already
     * @throws NoSuchAttributeDefinitionException
     *         if attr. def. is not valid
     * @throws NoSuchGuidException
     *          if guid name is not valid
     * @throws ValueTooLongException if the attrName is longer than
     *                               column size in DB schema
     * @see ColumnSizes
     * @see getColumnSizes( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     */

    virtual const std::string
    setGuidAttribute(const std::string& guidName,
		     const std::string& attrDefnName,
		     const std::string& attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchGuidException,
	    ValueTooLongException,
	    CommunicationException);
			
    /** Returns the old guid attribute, or <code>empty</code> string
     * if no old attribute exists
     *
     * @param guidName the guid name
     * @param attrDefnName the attribute definition name
     * @param attrValue the new attribute value
     * @return and old attribute value if exists already
     * @throws NoSuchAttributeDefinitionException if attr. def. is not valid
     * @throws NoSuchGuidException if guid name is not valid
     * @throws ValueTooLongException if the attrName is longer than
     *                               column size in DB schema
     * @see ColumnSizes
     * @see getColumnSizes( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string
    setStringGuidAttribute( const std::string& guidName,
			    const std::string& attrDefnName,
			    const std::string& attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchGuidException,
	    ValueTooLongException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);

    virtual int
    setIntGuidAttribute( const std::string& guidName,
			 const std::string& attrDefnName,
			 int                attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchGuidException,
	    ValueTooLongException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);

    virtual float
    setFloatGuidAttribute( const std::string& guidName,
			   const std::string& attrDefnName,
			   float              attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchGuidException,
	    ValueTooLongException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);

    /** Returns the guid attribute, or <code>empty</code> string
     * if no attribute exists
     *
     * @param guidName the guid name
     * @param attrDefnName the attribute definition name
     * @return an attribute value if exists
     * @throws NoSuchAttributeDefinitionException
     *         if attr. def. is not valid
     * @throws NoSuchGuidException
     *         if guid name is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string
    getGuidAttribute(const std::string& guidName,
		     const std::string& attrDefnName) const
      throw(NoSuchGuidException,
	    NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Returns the guid attribute, or <code>empty</code> string
     * if no attribute exists
     *
     * @param guidName the guid name
     * @param attrDefnName the attribute definition name
     * @return an attribute value if exists
     * @throws NoSuchAttributeDefinitionException if attr. def. is not valid
     * @throws NoSuchGuidException if guid name is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string
    getStringGuidAttribute( const std::string& guidName,
			    const std::string& attrDefnName) const
      throw(NoSuchGuidException,
	    NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
			
    virtual int
    getIntGuidAttribute( const std::string& guidName,
			 const std::string& attrDefnName) const
      throw(NoSuchGuidException,
	    NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
			
    virtual float
    getFloatGuidAttribute( const std::string& guidName,
			   const std::string& attrDefnName) const
      throw(NoSuchGuidException,
	    NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
			
    /** Removes the attribute for a given guid
     *
     * @param guidName the guid name
     * @param attrDefnName the attribute definition name
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    removeGuidAttribute(const std::string& guidName,
			const std::string& attrDefnName)
      throw(NoSuchGuidException, NoSuchAttributeDefinitionException, CommunicationException);
			
    /** Returns the old alias attribute, or <code>empty</code>
     * string if no old attribute exists
     *
     * @param aliasName the alias name
     * @param attrDefnName the attribute definition name
     * @param attrValue the new attribute value
     * @return and old attribute value if exists already
     * @throws NoSuchAttributeDefinitionException
     *         if attr. def. is not valid
     * @throws NoSuchAliasException if guid name is not valid
     * @throws ValueTooLongException if the attrName is longer than
     *                               column size in DB schema
     * @see ColumnSizes
     * @see getColumnSizes( ColumnSizes & )
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string
    setAliasAttribute(const std::string& aliasName,
		      const std::string& attrDefnName,
		      const std::string& attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchAliasException,
	    ValueTooLongException,
	    CommunicationException);
			
    virtual const std::string
    setStringAliasAttribute( const std::string& aliasName,
			     const std::string& attrDefnName,
			     const std::string& attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchAliasException,
	    ValueTooLongException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
			
    virtual int
    setIntAliasAttribute( const std::string& aliasName,
			  const std::string& attrDefnName,
			  int                attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchAliasException,
	    ValueTooLongException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);

    virtual float
    setFloatAliasAttribute( const std::string& aliasName,
			    const std::string& attrDefnName,
			    float              attrValue)
      throw(NoSuchAttributeDefinitionException,
	    NoSuchAliasException,
	    ValueTooLongException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);

    /** Returns the alias attribute, or <code>empty</code> string
     * if no attribute exists
     *
     * @param aliasName the alias name
     * @param attrDefnName the attribute definition name
     * @return an attribute value if exists
     * @throws NoSuchAttributeDefinitionException
     *         if attr. def. is not valid
     * @throws NoSuchAliasException
     *         if guid name is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string
    getAliasAttribute(const std::string& aliasName,
		      const std::string& attrDefnName) const 
      throw(NoSuchAliasException,
	    NoSuchAttributeDefinitionException,
	    CommunicationException);
    virtual const std::string
    getStringAliasAttribute( const std::string& aliasName,
			     const std::string& attrDefnName) const 
      throw(NoSuchAliasException,
	    NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
      
    virtual int
    getIntAliasAttribute( const std::string& aliasName,
			  const std::string& attrDefnName) const 
      throw(NoSuchAliasException,
	    NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
      
    virtual float
    getFloatAliasAttribute( const std::string& aliasName,
			    const std::string& attrDefnName) const 
      throw(NoSuchAliasException,
	    NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException,
	    CommunicationException);
			
    /** Removes the attribute for a given alias
     *
     * @param aliasName the alias name
     * @param attrDefnName the attribute definition name
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    removeAliasAttribute(const std::string& aliasName,
			 const std::string& attrDefnName)
      throw(NoSuchAliasException, NoSuchAttributeDefinitionException, CommunicationException);
			
    /** Set the number of mappings that will be displayed by default.
     * (for the getMappingsByGuid and getMappingsByPfn methods)..
     *
     * @param  resultLength the number of mappings that will be displayed
     *         by default
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    setDefaultResultLength(int resultLength)
      throw(CommunicationException);
			
    /** Return the number of mappings that will be displayed by default.
     * (for the getMappingsByGuid and getMappingsByPfn methods).
     *
     * @return the number of mappings displayed by default.
     * @throws CommunicationException in case of an unknown server error
     */
    virtual int
    getDefaultResultLength() const
      throw(CommunicationException);
			
    /** Return the DB schema defined column sizes for guid, alias and
     * attr. def. names
     * @param cs the structure to be filled by the corresponding values
     * @see ColumnSizes
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void getColumnSizes( ColumnSizes& cs ) const
      throw(CommunicationException);

    /** Return the mappings for a list of guids.  If some of the guids do not
     * exist, this is not a problem.  If there are no results, an empty array
     * is returned
     * @param guids the list of guids, which may be empty, but may not be
     * <code>null</code>
     * @param mappings pointer to a vector of Mapping objects to return
     *        the results in
     */
    virtual void
    getMappingsWithAttributes(const std::vector<std::string>& guids,
                              std::vector<Mapping>& mappings) const
      throw(CommunicationException);


    /** Add a set of guid to alias mappings, with optional guid and  alias attributes to
     * the database.  If any problem occurs when inserting entries or
     * attributes (alias already exists, inserting an attribute value for an
     * attribute which doesn't exist, inserting the wrong type of attribute
     * value) the entire transaction is rolled back, and a fault thrown
     *
     * @param mappings a vector of mappings
     */
    virtual void 
    setMappingsWithAttributes(const std::vector<Mapping>& mappings) const
      throw(CommunicationException, NoSuchAttributeDefinitionException,
	    UnappropriateAttributeTypeException, AliasExistsException);

    /** Return the guid to logical filename mappings which
     * match the given guid name pattern
     * The result will be an empty  if no mapppings are found.
     *
     * @param guidPattern the guid pattern
     * @param mappings reference to a vector of GuidAliasPair_t
     *                 to return the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuid(const std::string& guidPattern,
		      GaMap& mappings) const
      throw(CommunicationException);
			
    /** Return the guid to physical filename mappings which contain a guid
     * that matches the given guidPattern. Return only the first mappings 
     * (no more than the resultLength parameter).
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param guidPattern The GUID pattern we are searching for
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings pointer to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuid(const std::string& guidPattern,
		      int          resultLength,
		      GaMap&       mappings    ) const
      throw(CommunicationException);
			
    /** Return the guid to physical filename mappings which contain a guid
     * that matches the given guidPattern. Return only a certain number of mappings.
     * Return We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param guidPattern The GUID pattern we are searching for
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings pointer to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuid(const std::string&  guidPattern,
		      int           resultLength,
		      int           resultStart,
		      GaMap&        mappings    ) const
      throw(CommunicationException);
			
    /** Return the guid to logical filename mappings which
     * match the given alias name pattern
     * The result will be an empty  if no mapppings are found.
     *
     * @param aliasPattern the alias pattern
     * @param mappings reference to a vector of GuidAliasPair_t
     *                 to return the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAlias(const std::string& aliasPattern,
		       GaMap& mappings) const
      throw(CommunicationException);
			
    /** Return the guid to logical filename mappings which
     * match the given alias name pattern
     * The result will be an empty  if no mapppings are found.
     *
     * @param aliasPattern the alias pattern
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings reference to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAlias(const std::string&  aliasPattern,
		       int           resultLength,
		       GaMap&        mappings) const
      throw(CommunicationException);
			
    /** Return the guid to logical filename mappings which
     * match the given alias name pattern
     * The result will be an empty  if no mapppings are found.
     *
     * @param aliasPattern the alias pattern
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings reference to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAlias(const std::string&  aliasPattern,
		       int           resultLength,
		       int           resultStart,
		       GaMap&        mappings) const
      throw(CommunicationException);
			
    /** Return the guid to logical filename mappings which have
     * a given attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     *
     * @param name the name of the attribute
     * @param value the pattern value of attribute to search for
     * @param mappings reference to a vector of GuidAliasPair_t
     *        to return the results in
     * @throws NoSuchAttributeDefinitionException
     *         if attr. def. is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuidAttribute(const std::string& name,
			       const std::string& value,
			       GaMap&             mappings) const
      throw(NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Return the guid to logical filename mappings which have
     * a given attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     *
     * @param name the name of the attribute
     * @param value the pattern value of attribute to search for
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings reference to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws NoSuchAttributeDefinitionException if attr. def. is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuidAttribute(const std::string& name,
			       const std::string& value,
			       int                resultLength,
			       GaMap&             mappings) const
      throw(NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Return the guid to logical filename mappings which have
     * a given attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     *
     * @param name the name of the attribute
     * @param value the pattern value of attribute to search for
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings reference to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws NoSuchAttributeDefinitionException if attr. def. is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByGuidAttribute(const std::string& name,
			       const std::string& value,
			       int                resultLength,
			       int                resultStart,
			       GaMap&             mappings) const
      throw(NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Return the guid to logical filename mappings which have
     * a given attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     *
     * @param name the name of the attribute
     * @param value the pattern value of attribute to search for
     * @param mappings reference to a vector of GuidAliasPair_t
     *                 to return the results in
     * @throws NoSuchAttributeDefinitionException
     *         if attr. def. is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAliasAttribute(const std::string& name,
				const std::string& value,
				GaMap& mappings) const
      throw(NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Return the guid to logical filename mappings which have
     * a given attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     *
     * @param name the name of the attribute
     * @param value the pattern value of attribute to search for
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings reference to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws NoSuchAttributeDefinitionException if attr. def. is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAliasAttribute(const std::string& name,
				const std::string& value,
				int                resultLength,
				GaMap&             mappings) const
      throw(NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Return the guid to logical filename mappings which have
     * a given attribute name=value pair.
     * The result will be an empty  if no mapppings are found.
     *
     * @param name the name of the attribute
     * @param value the pattern value of attribute to search for
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings reference to a vector of GuidAliasPair_t to return
     *        the results in
     * @throws NoSuchAttributeDefinitionException if attr. def. is not valid
     * @throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAliasAttribute(const std::string& name,
				const std::string& value,
				int                resultLength,
				int                resultStart,
				GaMap&             mappings) const
      throw(NoSuchAttributeDefinitionException,
	    CommunicationException);
			
    /** Retrieve mappings by SQL query on attributes
     * @param SQLQuery     The query
     * @param resultLength The number of hits per call
     * @param mappings     The map of GUIDs -> aliases
     * throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAttribute( const std::string& SQLQuery, 
			    int                resultLength,
			    GaMap&             mappings ) const
      throw(CommunicationException);
      
    /** Retrieve mappings by SQL query on attributes
     * @param SQLQuery     The query
     * @param resultLength The number of hits per call
     * @param resultStart  The rank of the first mapping that will be displayed
     * @param mappings     The map of GUIDs -> aliases
     * throws CommunicationException in case of an unknown server error
     */
    virtual void
    getMappingsByAttribute( const std::string& SQLQuery, 
			    int                resultLength, 
			    int                resultStart,
			    GaMap&             mappings ) const
      throw(CommunicationException);

    /** Return the guid to alias mappings which the PFNs
     * corresponding to the given conditions, along with their attributes.

     * Return only a certain number of mappings.
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param SQLQuery The attributes conditions
     * @param resultLength The number of of mappings that will be displayed
     * @param mappings The mappings to be filled into
     */
    virtual void
    getMappingsWithAttributes( const std::string& SQLQuery, 
			       int resultLength,
			       std::vector<Mapping>& mappings) const
      throw(NoSuchAttributeDefinitionException,
	    InvalidQueryException,
	    CommunicationException);

    /** Return the guid to alias mappings which the PFNs
     * corresponding to the given conditions, along with their attributes.

     * Return only a certain number of mappings.
     * We assume unix regexp semantics.
     * The result will be an empty set if no mapppings are found.
     *
     * @param SQLQuery The attributes conditions
     * @param resultLength The number of of mappings that will be displayed
     * @param resultStart The rank of the first mapping that will be displayed
     * @param mappings The mappings to be filled into
     */
    virtual void
    getMappingsWithAttributes( const std::string& SQLQuery, 
			       int resultLength, 
			       int resultStart,
			       std::vector<Mapping>& mappings) const
      throw(NoSuchAttributeDefinitionException,
	    InvalidQueryException,
	    CommunicationException);


    /** Return RMC server version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getVersion() const
      throw(CommunicationException);
			
    /** Return RMC server interface version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getInterfaceVersion() const
      throw(CommunicationException);
			
    /** Return RMC server schema version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getSchemaVersion() const
      throw(CommunicationException);
			
    /** Check if a server is alive. It returns a stats string.
     * If everything is ok, this will start with <code>OK</code> else
     * it will start with <code>FAIL</code>.
     * @return the stats string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string ping() const
      throw(CommunicationException);			
			
  private:
    //
    // helper methods
    //

    void
    dump_soap(int ret, const struct soap* soap) const;

    void
    dump_soap_error(int ret, const struct soap* const_soap) const;

    void
    handleGenericFaults(int ret, const struct soap* soap) const;

    const char*
    endpoint() const;

    //
    // member variables
    //
    const std::string m_endpoint;
    struct soap*      m_soap;

#ifdef WITH_WP2_SECURITY
    /* sorry, had to loosen the visibility of soap()
     * a bit. Could try protected and friend decls as well.
     */
  public:
#endif
    struct soap*
    soap() const;

#ifdef WITH_WP2_SECURITY
    /* For defining the desired role and policy after the lrc
     * object has laready been created
     */
    int addRoleAndPolicy(char *role,
			 char *policy,
			 char *defaultrole,
			 char *defaultpolicy);

    void clearRoleAndPolicy();

#endif
  
  };
}

#endif // ReplicaMetadataCatalogImpl_H

// EOF ReplicaMetadataCatalogImpl.h

