/* -*- C++ -*-
 *
 * ReplicationExceptions.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */
#ifndef RMCReplicationExceptions_H
#define RMCReplicationExceptions_H 1

#include <exception>
#include <string>


/** 
 *
 * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
 * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
 * @version $Id: ReplicationExceptions.h,v 1.6 2004/06/27 15:42:10 jamesc Exp $
 */
namespace EdgReplicaMetadataCatalog {

  /**
   * A simple top-level exception class for Replication Exceptions.
   *
   */
  class ReplicationException : public std::exception {
  public:

  public:
    ReplicationException() throw() 
      : m_faultCode("FAULTABLE") {
    }
    ReplicationException(const std::string& what) throw() 
      : m_what (what), m_faultCode("INTERNAL") {
    }
    ReplicationException(const std::string& what,
			 const std::string& faultCode) throw() 
      : m_what (what), m_faultCode(faultCode) {
    }
    const std::string& faultCode() const throw() {
      return m_faultCode;
    }
    virtual const char* what () const throw() {
      return m_what.c_str ();
    }
    virtual ~ReplicationException() throw(){
    }

  private:
    std::string m_what;
    std::string m_faultCode;

  };
      

  /**
   * An exception thrown when we don't implement a method yet
   */
  class NotImplementedException : public ReplicationException {
  public: 
    NotImplementedException(const std::string& method) 
      : ReplicationException("Not Implemented: " + method),
        m_method(method) {
    }
    virtual ~NotImplementedException() throw(){
    }
    const std::string& getMethod() const {
      return m_method;
    }
      
  private:
    const std::string m_method;
  };

  /**
   * A exception thrown when ... .
   */
  class EmptyAttributeNameException : public ReplicationException {
  public: 
    EmptyAttributeNameException(const std::string& message) 
      : ReplicationException(message) {
    }
    virtual ~EmptyAttributeNameException() throw(){
    }
  };
  
  /**
   * A exception thrown when ... .
   */
  class NoSuchGuidException : public ReplicationException {
  public: 
    NoSuchGuidException(const std::string& guid) 
      : ReplicationException("No Such Guid: " + guid, "NOSUCHGUID"),
        m_guid(guid) {
    }
    virtual ~NoSuchGuidException() throw() {
    }
    const std::string& getGuid() const {
      return m_guid;
    }

  private:
    const std::string m_guid;
  };

  class NoSuchAliasException : public ReplicationException {
  public: 
    NoSuchAliasException(const std::string& alias) 
      : ReplicationException("No Such Alias: " + alias, "NOSUCHALIAS"),
        m_alias(alias) {
    }
    virtual ~NoSuchAliasException() throw(){
    }
    const std::string& getAlias() const {
      return m_alias;
    }
      
  private:
    const std::string m_alias;
  };

  class AliasExistsException : public ReplicationException {
  public: 
    AliasExistsException(const std::string& alias) 
      : ReplicationException("Alias Exists: " + alias, "ALIASEXISTS"),
        m_alias(alias) {
    }
    virtual ~AliasExistsException() throw(){
    }
    const std::string& getAlias() const {
      return m_alias;
    }

  private:
    const std::string m_alias;
  };

  /**
   * A exception thrown when ... .
   */
  class AttributeExistsException : public ReplicationException {
  public:  
    AttributeExistsException(const std::string& message) 
      : ReplicationException(message, "ATTRIBUTEEXISTS") {
    }
    virtual ~AttributeExistsException() throw() {
    }
  };

  /**
   * A exception thrown when ... .
   */
  class AttributeDefinitionExistsException : public ReplicationException {
  public: 
    AttributeDefinitionExistsException(const std::string& message) 
      : ReplicationException(message, "ATTRIBUTEDEFNEXISTS") {
    }
    virtual ~AttributeDefinitionExistsException() throw(){
    }
  };

  /**
   * A exception thrown when ... .
   */
  class NoSuchAttributeDefinitionException : public ReplicationException {
  public: 
    NoSuchAttributeDefinitionException(const std::string& message) 
      : ReplicationException(message, "NOSUCHATTRIBUTEDEFN") {
    }
    virtual ~NoSuchAttributeDefinitionException() throw(){
    }
  };

  /**
   * A exception thrown when ... .
   */
  class NoSuchAttributeException : public ReplicationException {
  public: 
    NoSuchAttributeException(const std::string& message) 
      : ReplicationException(message, "NOSUCHATTRIBUTE") {
    }
    virtual ~NoSuchAttributeException() throw(){
    }
  };

  /**
   * A exception thrown when ... .
   */
  class AliasesExistException : public ReplicationException {
  public: 
    // construct a new AliasesExistException 
    AliasesExistException(const std::string& guid) 
      : ReplicationException("Aliases Exist for guid : " + guid, "ALIASESEXISTS"),
        m_guid(guid) {
    }
    virtual ~AliasesExistException() throw(){
    }
    const std::string& getGuid() const {
      return m_guid;
    }
      
  private:
    const std::string m_guid;
  };

  class HasAliasesException : public ReplicationException {
  public: 
    HasAliasesException(const std::string& guid) 
      : ReplicationException("No Such Guid: " + guid, "HASALIASES"),
        m_guid(guid) {
    }
    virtual ~HasAliasesException() throw(){
    }
    const std::string& getGuid() const {
      return m_guid;
    }
			
  private:
    const std::string m_guid;
  };

  /**
   * An exception thrown when too long name is to be stored
   */
  class ValueTooLongException : public ReplicationException {
  public:
    ValueTooLongException( const std::string& message )
      : ReplicationException(message, "TOOLONGNAME") {
    }
    virtual ~ValueTooLongException() throw() {
    }
  };
  
  /**
   * An exception thrown when invalid attribute type is used
   */
  class UnappropriateAttributeTypeException : public ReplicationException {
  public:
    UnappropriateAttributeTypeException( const std::string& message )
      : ReplicationException(message, "ATTRTYPENOTGOODFORATTRNAME") {
    }
    virtual ~UnappropriateAttributeTypeException() throw() {
    }
  };

  /**
   * An exception thrown when invalid query is applied
   */
  class InvalidQueryException : public ReplicationException {
  public:
    InvalidQueryException( const std::string& message )
      : ReplicationException(message, "INVALIDQUERY") {
    }
    virtual ~InvalidQueryException() throw() {
    }
  };
	
  /**
   * An exception thrown when the attr type is unsupported
   */
  class UnsupportedAttributeTypeException : public ReplicationException {
  public:
    UnsupportedAttributeTypeException( const std::string& message )
      : ReplicationException(message, "NOSUCHATTRTYPE") {
    }
    virtual ~UnsupportedAttributeTypeException() throw() {
    }
  };

  /**
   * An exception thrown when...
   */
  class CommunicationException : public ReplicationException {
  public: 
    CommunicationException() 
      : ReplicationException() {
    }
    CommunicationException(const std::string& message) 
      : ReplicationException(message, "CONNECTION") {
    }
    virtual ~CommunicationException() throw(){
    }
  };

  /** A dispatch method for the soap faults that will throw the
   * appropriate language exception
   * @param fault fault string, as defined by the LRC SOAP API
   * @param detail fault specific detail information
   */
  void throwReplicationException(const std::string& fault, const std::string& detail);
}


#endif // ReplicationExceptions_H

