/* -*- C++ -*-
 *
 * ReplicaOptimization.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 *
 * $Id: ReplicaOptimization.h,v 1.12 2004/06/16 11:27:22 jamesc Exp $
 */


#ifndef ReplicaOptimization_H
#define ReplicaOptimization_H 1

/**
 * @author <a href="mailto:Kurt.Stockinger@cern.ch">Kurt Stockinger</a>
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
 */


#include <string>
#include <vector>

#include "EdgReplicaOptimization/ReplicationExceptions.h"

struct soap;
struct SOAP_ENV_Fault;

namespace EdgReplicaOptimization {
  struct tns1__AccessCost;
  struct tns1__BestNetworkCost;
  
  /** 
   */

  class AccessCost {
  public:
    typedef std::vector<std::string>            BestSFN_t;
    
  private:
    std::string         ceId;
    BestSFN_t           bestSFN;
    float       totalTime;
    float       sizeToBeReplicated;
    
  public:
    AccessCost(tns1__AccessCost &soapAccessCost);
    const std::string & getCeId() { return ceId; }
    const BestSFN_t & getBestSFN() { return bestSFN; }
    float getTotalTime() { return totalTime; }
    float getSizeToBeReplicated() { return sizeToBeReplicated; }
  };
  
  class BestNetworkCost {
  public:
    typedef std::vector<std::string>            BestSFN_t;
    
  private:
    std::string         bestSFN;
    float       time;
    
  public:
    BestNetworkCost() {}
    void setBNC(tns1__BestNetworkCost &soapBestNetworkCost);
    const std::string & getBestSFN() { return bestSFN; }
    float getTime() { return time; }
  };


  class ROSFile {
  private:
    std::string name;
    std::vector<std::string> replicas;
    float sizeMB;
    
  public:
    typedef std::vector<std::string>            Replicas_t;
    ROSFile() {}
    const std::string& getName() { return name; }
    const Replicas_t& getReplicas() { return replicas; }
    float getSizeMB() { return sizeMB; }

    void setName(const std::string &_name) {
      name = _name;
    }
    
    void setReplicas(const Replicas_t & _replicas) {
      replicas = _replicas;
    }
    
    void setSizeMB( const float _sizeMB) {
      sizeMB = _sizeMB;
    }
  };

  class ROSComputingElement {
  private:
    std::string host;
    std::string closeSE;
    
  public:
    ROSComputingElement() {}
    const std::string& getCEHost() { return host; }
    const std::string& getCloseSE() { return closeSE; }

    void setCEHost(const std::string &_host) {
      host = _host;
    }
      
    void setCloseSEs(const std::string & _closeSE) {
      closeSE = _closeSE;
    }
  };
  
  typedef std::vector<ROSFile> ROSFiles_t;
  typedef std::vector<ROSComputingElement> ROSComputingElements_t;
  typedef std::vector<std::string> Protocols_t;
  typedef std::vector<AccessCost> AccessCosts_t;

  class ReplicaOptimization
  {  
  public:
                        
    /** 
     * create a new ReplicaOptimization object
     * @param url     the URL to look for the catalog
     * @param timeOut the timeout after which gSOAP gives up and
     *                returns control back to client app, default is 30 seconds,
     *                if positive means seconds and if negative means microseconds
     */
                        
    ReplicaOptimization(const std::string& url,
                        int timeOut=30
#ifdef WITH_WP2_SECURITY
                        ,char *passphrase = NULL,
                        char *credentials = NULL,
                        char *cacertdir = NULL,
                        char *rseedfile = NULL,
                        char *role = NULL,
                        char *policy = NULL,
                        char *defaultRole = NULL,
                        char *defaultPolicy = NULL
#endif
                        ) {}

#ifdef WITH_WP2_SECURITY
    virtual int addRoleAndPolicy(char *role,
                                 char *policy,
                                 char *defaultrole,
                                 char *defaultpolicy) = 0;
      
    virtual void clearRoleAndPolicy() = 0;
#endif
                        
    /** Standard virtual destructor
     */
    virtual ~ReplicaOptimization() {};
                        
    /** get the URL this ReplicaOptimization resides at
     *
     * @return the URL
     */
    virtual const std::string& getUrl() const =0;
                        
                        
    /** Return ROS server version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getVersion() const
      throw(CommunicationException) =0;
      
    /** Return ROS server interface version as std::string
     * @throws CommunicationException in case of an unknown server error
     */
    virtual const std::string getInterfaceVersion() const
      throw(CommunicationException) =0;
      
    virtual bool ping() = 0;
                        
    virtual void getAccessCost(ROSFiles_t &files, 
                               ROSComputingElements_t &ce, 
                               Protocols_t &protocol,
                               AccessCosts_t &accessCost) = 0;
                        
    virtual void getBestNetworkCost(ROSFile &file,
                                    std::string &destinationSE,
                                    BestNetworkCost &networkCost) = 0;
  };
}

#endif // ReplicaOptimization_H

